
package rice.p2p.scribe.messaging;

import rice.*;
import rice.p2p.commonapi.*;
import rice.p2p.scribe.*;

/**
 * @(#) ScribeMessage.java
 *
 * This class the abstraction of a message used internally by Scribe.
 *
 * @version $Id: ScribeMessage.java 2302 2005-03-11 00:58:26Z jeffh $
 *
 * @author Alan Mislove
 */
public abstract class ScribeMessage implements Message {
  
  // serialver for backward compatibility
  private static final long serialVersionUID = 4593674882226544604L;

  // the source of this message
  protected NodeHandle source;

  // the topic of this message
  protected Topic topic;

  /**
   * Constructor which takes a unique integer Id
   *
   * @param source The source address
   * @param topic The topic
   */
  protected ScribeMessage(NodeHandle source, Topic topic) {
    this.source = source;
    this.topic = topic;
  }
  
  /**
   * Method which should return the priority level of this message.  The messages
   * can range in priority from 0 (highest priority) to Integer.MAX_VALUE (lowest) -
   * when sending messages across the wire, the queue is sorted by message priority.
   * If the queue reaches its limit, the lowest priority messages are discarded.  Thus,
   * applications which are very verbose should have LOW_PRIORITY or lower, and
   * applications which are somewhat quiet are allowed to have MEDIUM_PRIORITY or
   * possibly even HIGH_PRIORITY.
   *
   * @return This message's priority
   */
  public int getPriority() {
    return MEDIUM_HIGH_PRIORITY;
  }

  /**
   * Method which returns this messages' source address
   *
   * @return The source of this message
   */
  public NodeHandle getSource() {
    return source;
  }
  
  /**
   * Method which set this messages' source address
   *
   * @param source The source of this message
   */
  public void setSource(NodeHandle source) {
    this.source = source;
  }

  /**
   * Method which returns this messages' topic
   *
   * @return The topic of this message
   */
  public Topic getTopic() {
    return topic;
  }
  
}

